=== modified file 'NEWS'
--- NEWS	2006-06-10 23:29:41 +0000
+++ NEWS	2006-06-13 16:26:09 +0000
@@ -83,6 +83,10 @@
       the global revision graph but only that part leading up to a particular
       revision.  (Martin Pool, Robert Collins)
 
+    * Add a BzrDirFormat control_formats list which allows for control formats
+      that do not use '.bzr' to store their data - i.e. '.svn', '.hg' etc.
+      (Robert Collins, Jelmer Vernooij).
+
 bzr 0.8.2  2006-05-17
   
   BUG FIXES:

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-06-12 01:53:19 +0000
+++ bzrlib/bzrdir.py	2006-06-13 15:04:06 +0000
@@ -945,6 +945,12 @@
     _formats = {}
     """The known formats."""
 
+    _control_formats = []
+    """The registered control formats - .bzr, ....
+    
+    This is a list of BzrDirFormat objects.
+    """
+
     _lock_file_name = 'branch-lock'
 
     # _lock_class must be set in subclasses to the lock type, typ.
@@ -952,7 +958,18 @@
 
     @classmethod
     def find_format(klass, transport):
-        """Return the format registered for URL."""
+        """Return the format present at transport."""
+        for format in klass._control_formats:
+            try:
+                return format.probe_transport(transport)
+            except errors.NotBranchError:
+                # this format does not find a control dir here.
+                pass
+        raise errors.NotBranchError(path=transport.base)
+
+    @classmethod
+    def probe_transport(klass, transport):
+        """Return the .bzrdir style transport present at URL."""
         try:
             format_string = transport.get(".bzr/branch-format").read()
             return klass._formats[format_string]
@@ -1035,6 +1052,25 @@
         """
         return True
 
+    @classmethod
+    def known_formats(klass):
+        """Return all the known formats.
+        
+        Concrete formats should override _known_formats.
+        """
+        # There is double indirection here to make sure that control 
+        # formats used by more than one dir format will only be probed 
+        # once. This can otherwise be quite expensive for remote connections.
+        result = set()
+        for format in klass._control_formats:
+            result.update(format._known_formats())
+        return result
+    
+    @classmethod
+    def _known_formats(klass):
+        """Return the known format instances for this control format."""
+        return set(klass._formats.values())
+
     def open(self, transport, _found=False):
         """Return an instance of this format for the dir transport points at.
         
@@ -1058,6 +1094,17 @@
         klass._formats[format.get_format_string()] = format
 
     @classmethod
+    def register_control_format(klass, format):
+        """Register a format that does not use '.bzrdir' for its control dir.
+
+        TODO: This should be pulled up into a 'ControlDirFormat' base class
+        which BzrDirFormat can inherit from, and renamed to register_format 
+        there. It has been done without that for now for simplicity of
+        implementation.
+        """
+        klass._control_formats.append(format)
+
+    @classmethod
     def set_default_format(klass, format):
         klass._default_format = format
 
@@ -1069,6 +1116,14 @@
         assert klass._formats[format.get_format_string()] is format
         del klass._formats[format.get_format_string()]
 
+    @classmethod
+    def unregister_control_format(klass, format):
+        klass._control_formats.remove(format)
+
+
+# register BzrDirFormat as a control format
+BzrDirFormat.register_control_format(BzrDirFormat)
+
 
 class BzrDirFormat4(BzrDirFormat):
     """Bzr dir format 4.

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-06-13 12:55:45 +0000
+++ bzrlib/inventory.py	2006-06-13 16:26:09 +0000
@@ -77,31 +77,31 @@
     >>> i.path2id('')
     'TREE_ROOT'
     >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
-    InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
+    InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None)
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
-    InventoryFile('2323', 'hello.c', parent_id='123')
+    InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None)
     >>> shouldbe = {0: 'src', 1: pathjoin('src','hello.c')}
     >>> for ix, j in enumerate(i.iter_entries()):
     ...   print (j[0] == shouldbe[ix], j[1])
     ... 
-    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
-    (True, InventoryFile('2323', 'hello.c', parent_id='123'))
+    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT', revision=None))
+    (True, InventoryFile('2323', 'hello.c', parent_id='123', sha1=None, len=None))
     >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
     >>> i.add(InventoryFile('2324', 'bye.c', '123'))
-    InventoryFile('2324', 'bye.c', parent_id='123')
+    InventoryFile('2324', 'bye.c', parent_id='123', sha1=None, len=None)
     >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
-    InventoryDirectory('2325', 'wibble', parent_id='123')
+    InventoryDirectory('2325', 'wibble', parent_id='123', revision=None)
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
     >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
-    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    InventoryFile('2326', 'wibble.c', parent_id='2325', sha1=None, len=None)
     >>> i['2326']
-    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    InventoryFile('2326', 'wibble.c', parent_id='2325', sha1=None, len=None)
     >>> for path, entry in i.iter_entries():
     ...     print path
     ...     assert i.path2id(path)
@@ -393,11 +393,12 @@
         return 'unchanged'
 
     def __repr__(self):
-        return ("%s(%r, %r, parent_id=%r)"
+        return ("%s(%r, %r, parent_id=%r, revision=%r)"
                 % (self.__class__.__name__,
                    self.file_id,
                    self.name,
-                   self.parent_id))
+                   self.parent_id,
+                   self.revision))
 
     def snapshot(self, revision, path, previous_entries,
                  work_tree, commit_builder):
@@ -508,6 +509,7 @@
         self.kind = 'root_directory'
         self.parent_id = None
         self.name = u''
+        self.revision = None
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -679,6 +681,15 @@
         # in _read_tree_state
         self.executable = work_tree.is_executable(self.file_id, path=path)
 
+    def __repr__(self):
+        return ("%s(%r, %r, parent_id=%r, sha1=%r, len=%s)"
+                % (self.__class__.__name__,
+                   self.file_id,
+                   self.name,
+                   self.parent_id,
+                   self.text_sha1,
+                   self.text_size))
+
     def _forget_tree_state(self):
         self.text_sha1 = None
         self.executable = None
@@ -815,7 +826,7 @@
 
     >>> inv = Inventory()
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
-    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT')
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT', sha1=None, len=None)
     >>> inv['123-123'].name
     'hello.c'
 
@@ -832,7 +843,7 @@
     [u'hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
     >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
-    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678', sha1=None, len=None)
     """
     def __init__(self, root_id=ROOT_ID, revision_id=None):
         """Create or read an inventory.
@@ -849,6 +860,8 @@
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
         self.root = RootEntry(root_id)
+        # FIXME: this isn't ever used, changing it to self.revision may break
+        # things. TODO make everything use self.revision_id
         self.revision_id = revision_id
         self._byid = {self.root.file_id: self.root}
 
@@ -984,7 +997,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
-        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> '123' in inv
         True
         >>> '456' in inv
@@ -997,7 +1010,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123123', 'hello.c', ROOT_ID))
-        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT')
+        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> inv['123123'].name
         'hello.c'
         """
@@ -1070,7 +1083,7 @@
 
         >>> inv = Inventory()
         >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
-        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -1094,21 +1107,17 @@
         >>> i1 == i2
         True
         >>> i1.add(InventoryFile('123', 'foo', ROOT_ID))
-        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> i1 == i2
         False
         >>> i2.add(InventoryFile('123', 'foo', ROOT_ID))
-        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT', sha1=None, len=None)
         >>> i1 == i2
         True
         """
         if not isinstance(other, Inventory):
             return NotImplemented
 
-        if len(self._byid) != len(other._byid):
-            # shortcut: obviously not the same
-            return False
-
         return self._byid == other._byid
 
     def __ne__(self, other):

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2006-06-07 11:13:02 +0000
+++ bzrlib/revision.py	2006-06-13 13:24:40 +0000
@@ -51,6 +51,7 @@
         self._check_properties()
         self.parent_ids = []
         self.parent_sha1s = []
+        """Not used anymore - legacy from for 4."""
         self.__dict__.update(args)
 
     def __repr__(self):

=== modified file 'bzrlib/tests/bzrdir_implementations/__init__.py'
--- bzrlib/tests/bzrdir_implementations/__init__.py	2006-01-31 07:08:09 +0000
+++ bzrlib/tests/bzrdir_implementations/__init__.py	2006-05-30 13:02:37 +0000
@@ -38,12 +38,13 @@
     test_bzrdir_implementations = [
         'bzrlib.tests.bzrdir_implementations.test_bzrdir',
         ]
+    formats = BzrDirFormat.known_formats()
     adapter = BzrDirTestProviderAdapter(
         default_transport,
         # None here will cause a readonly decorator to be created
         # by the TestCaseWithTransport.get_readonly_transport method.
         None,
-        BzrDirFormat._formats.values())
+        formats)
     loader = TestLoader()
     adapt_modules(test_bzrdir_implementations, adapter, loader, result)
     return result

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-06-09 09:04:53 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-06-13 13:24:40 +0000
@@ -479,6 +479,71 @@
             bzrdir.BzrDirFormat.set_default_format(old_format)
 
 
+class NotBzrDir(bzrlib.bzrdir.BzrDir):
+    """A non .bzr based control directory."""
+
+    def __init__(self, transport, format):
+        self._format = format
+        self.root_transport = transport
+        self.transport = transport.clone('.not')
+
+
+class NotBzrDirFormat(bzrlib.bzrdir.BzrDirFormat):
+    """A test class representing any non-.bzr based disk format."""
+
+    def initialize_on_transport(self, transport):
+        """Initialize a new .not dir in the base directory of a Transport."""
+        transport.mkdir('.not')
+        return self.open(transport)
+
+    def open(self, transport):
+        """Open this directory."""
+        return NotBzrDir(transport, self)
+
+    @classmethod
+    def _known_formats(self):
+        return set([NotBzrDirFormat()])
+
+    @classmethod
+    def probe_transport(self, transport):
+        """Our format is present if the transport ends in '.not/'."""
+        if transport.has('.not'):
+            return NotBzrDirFormat()
+
+
+class TestNotBzrDir(TestCaseWithTransport):
+    """Tests for using the bzrdir api with a non .bzr based disk format.
+    
+    If/when one of these is in the core, we can let the implementation tests
+    verify this works.
+    """
+
+    def test_create_and_find_format(self):
+        # create a .notbzr dir 
+        format = NotBzrDirFormat()
+        dir = format.initialize(self.get_url())
+        self.assertIsInstance(dir, NotBzrDir)
+        # now probe for it.
+        bzrlib.bzrdir.BzrDirFormat.register_control_format(format)
+        try:
+            found = bzrlib.bzrdir.BzrDirFormat.find_format(
+                get_transport(self.get_url()))
+            self.assertIsInstance(found, NotBzrDirFormat)
+        finally:
+            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(format)
+
+    def test_included_in_known_formats(self):
+        bzrlib.bzrdir.BzrDirFormat.register_control_format(NotBzrDirFormat)
+        try:
+            formats = bzrlib.bzrdir.BzrDirFormat.known_formats()
+            for format in formats:
+                if isinstance(format, NotBzrDirFormat):
+                    return
+            self.fail("No NotBzrDirFormat in %s" % formats)
+        finally:
+            bzrlib.bzrdir.BzrDirFormat.unregister_control_format(NotBzrDirFormat)
+
+
 class NonLocalTests(TestCaseWithTransport):
     """Tests for bzrdir static behaviour on non local paths."""
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-06-13 12:55:45 +0000
+++ bzrlib/workingtree.py	2006-06-13 16:26:09 +0000
@@ -552,7 +552,7 @@
             if not path:
                 path = self._inventory.id2path(file_id)
             mode = os.lstat(self.abspath(path)).st_mode
-            return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
 
     @needs_write_lock
     def add(self, files, ids=None):

